#include "Ethernet_Demo_objects.h"
#include "Ethernet_Demo_resources.h"
#include "built_in.h"

// TFT module connections
char TFT_DataPort at LATE;
char TFT_DataPort_Direction at TRISE;
sbit TFT_RST at LATC1_bit;
sbit TFT_BLED at LATD2_bit;
sbit TFT_RS at LATB15_bit;
sbit TFT_CS at LATB14_bit;
sbit TFT_RD at LATD5_bit;
sbit TFT_WR at LATD4_bit;
sbit TFT_RST_Direction at TRISC1_bit;
sbit TFT_BLED_Direction at TRISD2_bit;
sbit TFT_RS_Direction at TRISB15_bit;
sbit TFT_CS_Direction at TRISB14_bit;
sbit TFT_RD_Direction at TRISD5_bit;
sbit TFT_WR_Direction at TRISD4_bit;
// End Glcd module connections

// Touch Panel module connections
sbit DriveX_Left at LATB13_bit;
sbit DriveX_Right at LATB11_bit;
sbit DriveY_Up at LATB12_bit;
sbit DriveY_Down at LATB10_bit;
sbit DriveX_Left_Direction at TRISB13_bit;
sbit DriveX_Right_Direction at TRISB11_bit;
sbit DriveY_Up_Direction at TRISB12_bit;
sbit DriveY_Down_Direction at TRISB10_bit;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
char PenDown;
void *PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

void PMPWaitBusy() {
  while(PMMODEbits.BUSY);
}

void SetIndex(unsigned short index) {
  TFT_RS = 0;
  PMDIN = index;
  PMPWaitBusy();
}

void WriteCommand( unsigned short cmd ) {
  TFT_RS = 1;
  PMDIN = cmd;
  PMPWaitBusy();
}

void WriteData(unsigned int _data) {
  TFT_RS = 1;
  PMDIN = _data;
  PMPWaitBusy();
}

static void InitializeTouchPanel() {
  AD1PCFG = 0xFFFF;
  PCFG12_bit = 0;
  PCFG13_bit = 0;
  // PMP setup
  PMMODE = 0;
  PMAEN  = 0;
  PMCON  = 0;  // WRSP: Write Strobe Polarity bit
  PMMODEbits.MODE = 2;     // Master 2
  PMMODEbits.WAITB = 0;
  PMMODEbits.WAITM = 1;
  PMMODEbits.WAITE = 0;
  PMMODEbits.MODE16 = 1;   // 16 bit mode
  PMCONbits.CSF = 0;
  PMCONbits.PTRDEN = 1;
  PMCONbits.PTWREN = 1;
  PMCONbits.PMPEN = 1;
  ADC1_Init();
  TFT_Set_Default_Mode();
  TP_TFT_Set_Default_Mode();
  TFT_Set_Active(SetIndex,WriteCommand,WriteData);
  TFT_Init(320, 240);

  TP_TFT_Init(320, 240, 13, 12);                                  // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(1000);                              // Set touch panel ADC threshold

  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}


/////////////////////////
  TScreen *CurrentScreen;

  TScreen                Screen1;
  TLabel                 Label1;
  char Label1_Caption[17] = "MikroElektronika";

  TImage                 Image1;
  TCircle                Circle1;
  TCircle                Circle2;
  TCircle                Circle3;
  TCircle                Circle4;
  TCircle                Circle5;
  TCircle                Circle6;
  TCircle                Circle7;
  TCircle                Circle8;
  TLabel                 *Screen0_Labels[1];
  TImage                 *Screen0_Images[1];
  TCircle                *Screen0_Circles[8];



static void InitializeObjects() {
  Screen1.Color                     = 0x0000;
  Screen1.Width                     = 320;
  Screen1.Height                    = 240;
  Screen1.LabelsCount               = 1;
  Screen1.Labels                    = Screen0_Labels;
  Screen1.ImagesCount               = 1;
  Screen1.Images                    = Screen0_Images;
  Screen1.CirclesCount              = 8;
  Screen1.Circles                   = Screen0_Circles;
  Screen1.ObjectsCount              = 10;
  Screen0_Labels[0]                 = &Label1;
  Screen0_Images[0]                 = &Image1;
  Screen0_Circles[0]                = &Circle1;
  Screen0_Circles[1]                = &Circle2;
  Screen0_Circles[2]                = &Circle3;
  Screen0_Circles[3]                = &Circle4;
  Screen0_Circles[4]                = &Circle5;
  Screen0_Circles[5]                = &Circle6;
  Screen0_Circles[6]                = &Circle7;
  Screen0_Circles[7]                = &Circle8;


  Label1.OwnerScreen    = &Screen1;
  Label1.Order          = 0;
  Label1.Left           = 40;
  Label1.Top            = 63;
  Label1.Width          = 250;
  Label1.Height         = 36;
  Label1.Visible        = 1;
  Label1.Active         = 1;
  Label1.Caption        = Label1_Caption;
  Label1.FontName       = Impact26x39;
  Label1.Font_Pos_Ver   = 60;
  Label1.Font_Color     = 0xF800;
  Label1.OnUpPtr        = 0;
  Label1.OnDownPtr      = 0;
  Label1.OnClickPtr     = 0;
  Label1.OnPressPtr     = 0;

  Image1.OwnerScreen    = &Screen1;
  Image1.Order          = 1;
  Image1.Left           = 23;
  Image1.Top            = 7;
  Image1.Width          = 279;
  Image1.Height         = 45;
  Image1.Picture_Type   = 0;
  Image1.Picture_Ratio  = 1;
  Image1.Picture_Name   = Image1_mikroe_logo;
  Image1.Visible        = 1;
  Image1.Active         = 1;
  Image1.OnUpPtr        = 0;
  Image1.OnDownPtr      = 0;
  Image1.OnClickPtr     = 0;
  Image1.OnPressPtr     = 0;

  Circle1.OwnerScreen     = &Screen1;
  Circle1.Order           = 2;
  Circle1.Left            = 19;
  Circle1.Top             = 128;
  Circle1.Radius          = 18;
  Circle1.Pen_Width       = 1;
  Circle1.Pen_Color       = 0x0000;
  Circle1.Visible         = 1;
  Circle1.Active          = 1;
  Circle1.Transparent     = 1;
  Circle1.Gradient        = 0;
  Circle1.Gradient_Orientation    = 0;
  Circle1.Gradient_Start_Color    = 0xFFFF;
  Circle1.Gradient_End_Color      = 0xC618;
  Circle1.Color           = 0xFFFF;
  Circle1.PressColEnabled = 0;
  Circle1.Press_Color     = 0x8410;
  Circle1.OnUpPtr        = 0;
  Circle1.OnDownPtr      = 0;
  Circle1.OnClickPtr     = Circle1Click;
  Circle1.OnPressPtr     = 0;

  Circle2.OwnerScreen     = &Screen1;
  Circle2.Order           = 3;
  Circle2.Left            = 92;
  Circle2.Top             = 128;
  Circle2.Radius          = 18;
  Circle2.Pen_Width       = 1;
  Circle2.Pen_Color       = 0x0000;
  Circle2.Visible         = 1;
  Circle2.Active          = 1;
  Circle2.Transparent     = 1;
  Circle2.Gradient        = 0;
  Circle2.Gradient_Orientation    = 0;
  Circle2.Gradient_Start_Color    = 0xFFFF;
  Circle2.Gradient_End_Color      = 0xC618;
  Circle2.Color           = 0xFFFF;
  Circle2.PressColEnabled = 0;
  Circle2.Press_Color     = 0x8410;
  Circle2.OnUpPtr        = 0;
  Circle2.OnDownPtr      = 0;
  Circle2.OnClickPtr     = Circle2Click;
  Circle2.OnPressPtr     = 0;

  Circle3.OwnerScreen     = &Screen1;
  Circle3.Order           = 4;
  Circle3.Left            = 165;
  Circle3.Top             = 128;
  Circle3.Radius          = 18;
  Circle3.Pen_Width       = 1;
  Circle3.Pen_Color       = 0x0000;
  Circle3.Visible         = 1;
  Circle3.Active          = 1;
  Circle3.Transparent     = 1;
  Circle3.Gradient        = 0;
  Circle3.Gradient_Orientation    = 0;
  Circle3.Gradient_Start_Color    = 0xFFFF;
  Circle3.Gradient_End_Color      = 0xC618;
  Circle3.Color           = 0xFFFF;
  Circle3.PressColEnabled = 0;
  Circle3.Press_Color     = 0x8410;
  Circle3.OnUpPtr        = 0;
  Circle3.OnDownPtr      = 0;
  Circle3.OnClickPtr     = Circle3Click;
  Circle3.OnPressPtr     = 0;

  Circle4.OwnerScreen     = &Screen1;
  Circle4.Order           = 5;
  Circle4.Left            = 238;
  Circle4.Top             = 128;
  Circle4.Radius          = 18;
  Circle4.Pen_Width       = 1;
  Circle4.Pen_Color       = 0x0000;
  Circle4.Visible         = 1;
  Circle4.Active          = 1;
  Circle4.Transparent     = 1;
  Circle4.Gradient        = 0;
  Circle4.Gradient_Orientation    = 0;
  Circle4.Gradient_Start_Color    = 0xFFFF;
  Circle4.Gradient_End_Color      = 0xC618;
  Circle4.Color           = 0xFFFF;
  Circle4.PressColEnabled = 0;
  Circle4.Press_Color     = 0x8410;
  Circle4.OnUpPtr        = 0;
  Circle4.OnDownPtr      = 0;
  Circle4.OnClickPtr     = Circle4Click;
  Circle4.OnPressPtr     = 0;

  Circle5.OwnerScreen     = &Screen1;
  Circle5.Order           = 6;
  Circle5.Left            = 41;
  Circle5.Top             = 184;
  Circle5.Radius          = 18;
  Circle5.Pen_Width       = 1;
  Circle5.Pen_Color       = 0x0000;
  Circle5.Visible         = 1;
  Circle5.Active          = 1;
  Circle5.Transparent     = 1;
  Circle5.Gradient        = 0;
  Circle5.Gradient_Orientation    = 0;
  Circle5.Gradient_Start_Color    = 0xFFFF;
  Circle5.Gradient_End_Color      = 0xC618;
  Circle5.Color           = 0xFFFF;
  Circle5.PressColEnabled = 1;
  Circle5.Press_Color     = 0x8410;
  Circle5.OnUpPtr        = 0;
  Circle5.OnDownPtr      = 0;
  Circle5.OnClickPtr     = Circle5Click;
  Circle5.OnPressPtr     = 0;

  Circle6.OwnerScreen     = &Screen1;
  Circle6.Order           = 7;
  Circle6.Left            = 117;
  Circle6.Top             = 184;
  Circle6.Radius          = 18;
  Circle6.Pen_Width       = 1;
  Circle6.Pen_Color       = 0x0000;
  Circle6.Visible         = 1;
  Circle6.Active          = 1;
  Circle6.Transparent     = 1;
  Circle6.Gradient        = 0;
  Circle6.Gradient_Orientation    = 0;
  Circle6.Gradient_Start_Color    = 0xFFFF;
  Circle6.Gradient_End_Color      = 0xC618;
  Circle6.Color           = 0xFFFF;
  Circle6.PressColEnabled = 1;
  Circle6.Press_Color     = 0x8410;
  Circle6.OnUpPtr        = 0;
  Circle6.OnDownPtr      = 0;
  Circle6.OnClickPtr     = Circle6Click;
  Circle6.OnPressPtr     = 0;

  Circle7.OwnerScreen     = &Screen1;
  Circle7.Order           = 8;
  Circle7.Left            = 193;
  Circle7.Top             = 184;
  Circle7.Radius          = 18;
  Circle7.Pen_Width       = 1;
  Circle7.Pen_Color       = 0x0000;
  Circle7.Visible         = 1;
  Circle7.Active          = 1;
  Circle7.Transparent     = 1;
  Circle7.Gradient        = 0;
  Circle7.Gradient_Orientation    = 0;
  Circle7.Gradient_Start_Color    = 0xFFFF;
  Circle7.Gradient_End_Color      = 0xC618;
  Circle7.Color           = 0xFFFF;
  Circle7.PressColEnabled = 1;
  Circle7.Press_Color     = 0x8410;
  Circle7.OnUpPtr        = 0;
  Circle7.OnDownPtr      = 0;
  Circle7.OnClickPtr     = Circle7Click;
  Circle7.OnPressPtr     = 0;

  Circle8.OwnerScreen     = &Screen1;
  Circle8.Order           = 9;
  Circle8.Left            = 269;
  Circle8.Top             = 184;
  Circle8.Radius          = 18;
  Circle8.Pen_Width       = 1;
  Circle8.Pen_Color       = 0x0000;
  Circle8.Visible         = 1;
  Circle8.Active          = 1;
  Circle8.Transparent     = 1;
  Circle8.Gradient        = 0;
  Circle8.Gradient_Orientation    = 0;
  Circle8.Gradient_Start_Color    = 0xFFFF;
  Circle8.Gradient_End_Color      = 0xC618;
  Circle8.Color           = 0xFFFF;
  Circle8.PressColEnabled = 1;
  Circle8.Press_Color     = 0x8410;
  Circle8.OnUpPtr        = 0;
  Circle8.OnDownPtr      = 0;
  Circle8.OnClickPtr     = Circle8Click;
  Circle8.OnPressPtr     = 0;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetLabel(index)               CurrentScreen->Labels[index]
#define GetImage(index)               CurrentScreen->Images[index]
#define GetCircle(index)              CurrentScreen->Circles[index]


void DrawLabel(TLabel *Alabel) {
  if (Alabel->Visible == 1) {
    TFT_Set_Font(Alabel->FontName, Alabel->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text(Alabel->Caption, Alabel->Left, Alabel->Top);
  }
}

static void DrawLabels() {
  int i;

  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    DrawLabel(GetLabel(i));
  }
}


void DrawImage(TImage *Aimage) {
  if (Aimage->Visible == 1) {
    if (Aimage->Picture_Type == 0) {
      TFT_Image(Aimage->Left, Aimage->Top, Aimage->Picture_Name, Aimage->Picture_Ratio);
    }
  }
}

static void DrawImages() {
  int i;

  for ( i = 0 ; i < CurrentScreen->ImagesCount ; i++ ) {
    DrawImage(GetImage(i));
  }
}


void DrawCircle(TCircle *Acircle) {
  if (Acircle->Visible == 1) {
    TFT_Set_Brush(Acircle->Transparent, Acircle->Color, Acircle->Gradient, Acircle->Gradient_Orientation,
                  Acircle->Gradient_Start_Color, Acircle->Gradient_End_Color);
    TFT_Set_Pen(Acircle->Pen_Color, Acircle->Pen_Width);
    TFT_Circle(Acircle->Left + Acircle->Radius,
               Acircle->Top + Acircle->Radius,
               Acircle->Radius);
  }
}

static void DrawCircles() {
  int i;

  for ( i = 0 ; i < CurrentScreen->CirclesCount ; i++ ) {
    DrawCircle(GetCircle(i));
  }
}

void DrawScreen(TScreen *aScreen) {
  unsigned short order;
  unsigned short label_idx;
  TLabel *local_label;
  unsigned short image_idx;
  TImage *local_image;
  unsigned short circle_idx;
  TCircle *local_circle;
  order = 0;
  label_idx = 0;
  image_idx = 0;
  circle_idx = 0;

  if ((display_width != aScreen->Width) || (display_height != aScreen->Height)) {
    TFT_BLED_Direction = 0;
    TFT_BLED           = 0;
    TFT_Init(aScreen->Width, aScreen->Height);
    TP_TFT_Init(aScreen->Width, aScreen->Height, 13, 12);                                  // Initialize touch panel
    TP_TFT_Set_ADC_Threshold(1000);                              // Set touch panel ADC threshold
    TFT_Fill_Screen(aScreen->Color);
    display_width = aScreen->Width;
    display_height = aScreen->Height;
    TFT_BLED           = 1;
  }
  else
    TFT_Fill_Screen(aScreen->Color);

  CurrentScreen = aScreen;


  while (order < CurrentScreen->ObjectsCount) {
    if (label_idx < CurrentScreen->LabelsCount) {
      local_label = GetLabel(label_idx);
      if (order == local_label->Order) {
        label_idx++;
        order++;
        DrawLabel(local_label);
      }
    }

    if (circle_idx < CurrentScreen->CirclesCount) {
      local_circle = GetCircle(circle_idx);
      if (order == local_circle->Order) {
        circle_idx++;
        order++;
        DrawCircle(local_circle);
      }
    }

    if (image_idx  < CurrentScreen->ImagesCount) {
      local_image = GetImage(image_idx);
      if (order == local_image->Order) {
        image_idx++;
        order++;
        DrawImage(local_image);
      }
    }

  }
}

static void Process_TP_Press(unsigned int X, unsigned int Y) {
  int i;
  TLabel *local_label;
  TLabel *exec_label;
  short label_order;
  TImage *local_image;
  TImage *exec_image;
  short image_order;
  TCircle *local_circle;
  TCircle *exec_circle;
  short circle_order;

  label_order         = -1;
  exec_label          = 0;
  image_order         = -1;
  exec_image          = 0;
  circle_order        = -1;
  exec_circle         = 0;

  // Labels
  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  // Images
  for ( i = 0 ; i < CurrentScreen->ImagesCount ; i++ ) {
    local_image = GetImage(i);
    if (local_image->Active == 1) {
      if (IsInsideObject(X, Y, local_image->Left, local_image->Top,
                         local_image->Width, local_image->Height) == 1) {
        image_order = local_image->Order;
        exec_image = local_image;
      }
    }
  }

  // Circles
  for ( i = 0 ; i < CurrentScreen->CirclesCount ; i++ ) {
    local_circle = GetCircle(i);
    if (local_circle->Active == 1) {
      if (IsInsideObject(X, Y, local_circle->Left, local_circle->Top,
                        (local_circle->Radius * 2), (local_circle->Radius * 2)) == 1) {
        circle_order = local_circle->Order;
        exec_circle = local_circle;
      }
    }
  }

  i = -1;
  if (label_order > i)
    i = label_order;
  if (image_order > i)
    i = image_order;
  if (circle_order > i)
    i = circle_order;

  if (i != -1) {
    if (i == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnPressPtr != 0) {
          exec_label->OnPressPtr();
          return;
        }
      }
    }

    if (i == image_order) {
      if (exec_image->Active == 1) {
        if (exec_image->OnPressPtr != 0) {
          exec_image->OnPressPtr();
          return;
        }
      }
    }

    if (i == circle_order) {
      if (exec_circle->Active == 1) {
        if (exec_circle->OnPressPtr != 0) {
          exec_circle->OnPressPtr();
          return;
        }
      }
    }

  }
}

static void Process_TP_Up(unsigned int X, unsigned int Y) {
  int i;
  TLabel *local_label;
  TLabel *exec_label;
  short label_order;
  TImage *local_image;
  TImage *exec_image;
  short image_order;
  TCircle *local_circle;
  TCircle *exec_circle;
  short circle_order;

  switch (PressedObjectType) {
    // Circle
    case 4: {
      if (PressedObject != 0) {
        exec_circle = (TCircle*)PressedObject;
        if (exec_circle->PressColEnabled == 1) {
          TFT_Set_Brush(exec_circle->Transparent, exec_circle->Color, exec_circle->Gradient, exec_circle->Gradient_Orientation,
                        exec_circle->Gradient_Start_Color, exec_circle->Gradient_End_Color);
          TFT_Set_Pen(exec_circle->Pen_Color, exec_circle->Pen_Width);
          TFT_Circle(exec_circle->Left + exec_circle->Radius,
                     exec_circle->Top + exec_circle->Radius,
                     exec_circle->Radius);
        }
        break;
      }
      break;
    }
  }

  label_order         = -1;
  exec_label          = 0;
  image_order         = -1;
  exec_image          = 0;
  circle_order        = -1;

  // Labels
  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  // Images
  for ( i = 0 ; i < CurrentScreen->ImagesCount ; i++ ) {
    local_image = GetImage(i);
    if (local_image->Active == 1) {
      if (IsInsideObject(X, Y, local_image->Left, local_image->Top,
                         local_image->Width, local_image->Height) == 1) {
        image_order = local_image->Order;
        exec_image = local_image;
      }
    }
  }

  // Circles
  for ( i = 0 ; i < CurrentScreen->CirclesCount ; i++ ) {
    local_circle = GetCircle(i);
    if (local_circle->Active == 1) {
      if (IsInsideObject(X, Y, local_circle->Left, local_circle->Top,
                        (local_circle->Radius * 2), (local_circle->Radius * 2)) == 1) {
        circle_order = local_circle->Order;
        exec_circle = local_circle;
      }
    }
  }

  i = -1;
  if (label_order > i)
    i = label_order;
  if (image_order > i)
    i = image_order;
  if (circle_order > i)
    i = circle_order;

  if (i != -1) {
  // Labels
    if (i == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnUpPtr != 0)
          exec_label->OnUpPtr();
        if (PressedObject == (void *)exec_label)
          if (exec_label->OnClickPtr != 0)
            exec_label->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Images
    if (i == image_order) {
      if (exec_image->Active == 1) {
        if (exec_image->OnUpPtr != 0)
          exec_image->OnUpPtr();
        if (PressedObject == (void *)exec_image)
          if (exec_image->OnClickPtr != 0)
            exec_image->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Circles
    if (i == circle_order) {
      if (exec_circle->Active == 1) {
        if (exec_circle->OnUpPtr != 0)
          exec_circle->OnUpPtr();
        if (PressedObject == (void *)exec_circle)
          if (exec_circle->OnClickPtr != 0)
            exec_circle->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

static void Process_TP_Down(unsigned int X, unsigned int Y) {
  int i;
  TLabel *local_label;
  TLabel *exec_label;
  short label_order;
  TImage *local_image;
  TImage *exec_image;
  short image_order;
  TCircle *local_circle;
  TCircle *exec_circle;
  short circle_order;

  label_order         = -1;
  exec_label          = 0;
  image_order         = -1;
  exec_image          = 0;
  circle_order        = -1;
  exec_circle         = 0;

  // Labels
  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  // Images
  for ( i = 0 ; i < CurrentScreen->ImagesCount ; i++ ) {
    local_image = GetImage(i);
    if (local_image->Active == 1) {
      if (IsInsideObject(X, Y, local_image->Left, local_image->Top,
                         local_image->Width, local_image->Height) == 1) {
        image_order = local_image->Order;
        exec_image = local_image;
      }
    }
  }

  // Circles
  for ( i = 0 ; i < CurrentScreen->CirclesCount ; i++ ) {
    local_circle = GetCircle(i);
    if (local_circle->Active == 1) {
      if (IsInsideObject(X, Y, local_circle->Left, local_circle->Top,
                        (local_circle->Radius * 2), (local_circle->Radius * 2)) == 1) {
        circle_order = local_circle->Order;
        exec_circle = local_circle;
      }
    }
  }

  i = -1;
  if (label_order > i)
    i = label_order;
  if (image_order > i)
    i = image_order;
  if (circle_order > i)
    i = circle_order;
  if (i != -1) {
    if (i == label_order) {
      if (exec_label->Active == 1) {
        PressedObject = (void *)exec_label;
        PressedObjectType = 2;
        if (exec_label->OnDownPtr != 0) {
          exec_label->OnDownPtr();
          return;
        }
      }
    }

    if (i == image_order) {
      if (exec_image->Active == 1) {
        PressedObject = (void *)exec_image;
        PressedObjectType = 3;
        if (exec_image->OnDownPtr != 0) {
          exec_image->OnDownPtr();
          return;
        }
      }
    }

    if (i == circle_order) {
      if (exec_circle->Active == 1) {
        if (exec_circle->PressColEnabled == 1) {
          TFT_Set_Brush(exec_circle->Transparent, exec_circle->Press_Color, exec_circle->Gradient, exec_circle->Gradient_Orientation,
                        exec_circle->Gradient_End_Color, exec_circle->Gradient_Start_Color);
          TFT_Set_Pen(exec_circle->Pen_Color, exec_circle->Pen_Width);
          TFT_Circle(exec_circle->Left + exec_circle->Radius,
                     exec_circle->Top + exec_circle->Radius,
                     exec_circle->Radius);
        }
        PressedObject = (void *)exec_circle;
        PressedObjectType = 4;
        if (exec_circle->OnDownPtr != 0) {
          exec_circle->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (TP_TFT_Press_Detect()) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
    if (TP_TFT_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Start_TP() {
  InitializeTouchPanel();

  // You can get calibration constants using touch panel calibration example
  TP_TFT_Set_Calibration_Consts(76, 907, 77, 915);    // Set calibration constants

  InitializeObjects();
  display_width = Screen1.Width;
  display_height = Screen1.Height;
  DrawScreen(&Screen1);
}